@@ -15,6 +15,7 @@
 import os
 import sys
 import errno
+import time
 
 from lockfile import (
     LockBase,
@@ -61,16 +62,36 @@ class PIDLockFile(LockBase):
             result = True
         return result
 
-    def acquire(self):
+    poll_interval = 0.1
+
+    def acquire(self, timeout=None):
         """ Acquire the lock.
 
-            Creates the PID file for this lock, or raises an error if
-            the lock was already held.
+            Creates the PID file for this lock, then returns None.
+
+            If the lock is already held, behaviour depends on the
+            `timeout` parameter:
+
+            * `timeout` is ``None``: poll every 0.1 seconds, waiting
+              for the lock indefinitely.
+
+            * `timeout` > 0: poll every 0.1 seconds, waiting for the
+              lock. After `timeout` seconds elapse without acquiring
+              the lock, raise an `AlreadyLocked` error.
+
+            * `timeout` <= 0: immediately raise an `AlreadyLocked`
+              error.
 
             """
-        if pidfile_exists(self.path):
-            error = AlreadyLocked()
-            raise error
+        if timeout is not None:
+            request_timestamp = time.time()
+            timeout_timestamp = request_timestamp + timeout
+        while pidfile_exists(self.path):
+            if timeout is not None:
+                if time.time() > timeout_timestamp:
+                    error = AlreadyLocked()
+                    raise error
+            time.sleep(self.poll_interval)
         try:
             write_pid_to_pidfile(self.path)
         except OSError:
